(function(context) {
  function isObj(val) {
    var type = typeof val;
    return val !== null && (type === 'object' || type === 'function');
  }

  var hasOwnProperty = Object.prototype.hasOwnProperty;
  var propIsEnumerable = Object.prototype.propertyIsEnumerable;

  function toObject(val) {
    if (val === null || val === undefined) {
      throw new TypeError('Cannot convert undefined or null to object');
    }

    return Object(val);
  }

  function assignKey(to, from, key) {
    var val = from[key];

    if (val === undefined || val === null) {
      return;
    }

    if (hasOwnProperty.call(to, key)) {
      if (to[key] === undefined || to[key] === null) {
        throw new TypeError('Cannot convert undefined or null to object (' + key + ')');
      }
    }

    if (!hasOwnProperty.call(to, key) || !isObj(val)) {
      to[key] = val;
    } else {
      to[key] = assign(Object(to[key]), from[key]);
    }
  }

  function assign(to, from) {
    if (to === from) {
      return to;
    }

    from = Object(from);

    for (var key in from) {
      if (hasOwnProperty.call(from, key)) {
        assignKey(to, from, key);
      }
    }

    if (Object.getOwnPropertySymbols) {
      var symbols = Object.getOwnPropertySymbols(from);

      for (var i = 0; i < symbols.length; i++) {
        if (propIsEnumerable.call(from, symbols[i])) {
          assignKey(to, from, symbols[i]);
        }
      }
    }

    return to;
  }

  context['deepAssign'] = function(target) {
    target = toObject(target);

    for (var s = 1; s < arguments.length; s++) {
      assign(target, arguments[s]);
    }

    return target;
  }

})(this);

/**
 * 游云南jsBridge
 * @param  {Object} context    当前上下文
 * @param  {[type]} nameSpace  命名空间
 * @param  {[type]} callType   android or ios
 */
(function(context, nameSpace, callType) {
  /**
   * 系统声明和可以调用的方法
   */
  var declaredPublicMethods = [
    'setGestureQuit',
    'getGestureQuit',
    'locationInfo',
    'setTitle',
    'closeWindow',
    'openWindow',
    'setActionBar',
    'login',
    'getAppData',
    'clearCache',
    'alysis',
    'showLoading',
    'sendAppData',
    'message',
    'getGallery'
  ];

  /**
   * 全局对象
   */
  var T = {};
  var E = {};

  /**
   * 一个数组用来存储回调函数
   */
  T.callBackQueue = [];

  /**
   * 临时变量
   */
  var S = Array.prototype.slice;
  // Android -  Mozilla/5.0 (Linux; Android 5.0.2; MI 2 Build/LRX22G) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/37.0.0.0 Mobile Safari/537.36 wii/2.0.2.0
  // iOS -
  var ua = navigator.userAgent;

  function getVersion() {
    var tem = ua.match(/yyn\/([\.\d]+)/i);
//    return tem && tem[1] ? tem[1] : "0.0.0.0";
    return "2.2.0.0";
  };

  function isAndroid() {
    return /android/i.test(ua) ? true : false;
  }

  function isIOS() {
    return /iPhone/i.test(ua) ? true : false;
  }

  /**
   * 版本比较
   */
  function versionCompare(v1, v2, options) {
    var lexicographical = options && options.lexicographical,
      zeroExtend = options && options.zeroExtend,
      v1parts = v1.split('.'),
      v2parts = v2.split('.');

    function isValidPart(x) {
      return (lexicographical ? /^\d+[A-Za-z]*$/ : /^\d+$/).test(x);
    }
    if (!v1parts.every(isValidPart) || !v2parts.every(isValidPart)) {
      return NaN;
    }
    if (zeroExtend) {
      while (v1parts.length < v2parts.length) v1parts.push("0");
      while (v2parts.length < v1parts.length) v2parts.push("0");
    }
    if (!lexicographical) {
      v1parts = v1parts.map(Number);
      v2parts = v2parts.map(Number);
    }
    for (var i = 0; i < v1parts.length; ++i) {
      if (v2parts.length == i) {
        return 1;
      }
      if (v1parts[i] == v2parts[i]) {
        continue;
      } else if (v1parts[i] > v2parts[i]) {
        return 1;
      } else {
        return -1;
      }
    }
    if (v1parts.length != v2parts.length) {
      return -1;
    }
    return 0;
  }

  /**
   * 异步回调处理函数
   */
  T.javascriptBridgeCallBack = function() {
    try {
      var params = S.call(arguments, 0);
      var val = params.shift();
      var index = parseInt(val);
      T.callBackQueue[index].apply(this, params);
    } catch (ex) {
      alert(ex);
    }
  };

  /**
   * 参数数据转化成所需的Json字符串
   */
  function arrayToJsonString(paramsArray) {
    var typeArray = [];
    var method = paramsArray.shift();
    var n = paramsArray.length;
    for (var i = 0; i < n; i++) {
      var params = paramsArray[i];
      var type = typeof params;
      if (params == null) {
        typeArray[typeArray.length] = "string";
      } else {
        typeArray[typeArray.length] = type;
      }
      if (type == "function") {
        var d = T.callBackQueue.length;
        T.callBackQueue[d] = params;
        paramsArray[i] = d;
      }
    }

    var jsonStr = JSON.stringify({
      method: method,
      types: typeArray,
      args: paramsArray,
      instanceName: nameSpace
    });

    return jsonStr;
  }

  /**
   * 通过prompt调用Java对象方法
   */
  function promptCall() {
    var re = {};
    try{
      var paramsArray = S.call(arguments, 0);
      // console.log(paramsArray);
      var jsonResult = prompt(arrayToJsonString(paramsArray));
      // console.log(jsonResult);
      var re = JSON.parse(jsonResult);
      // console.log(re);
      if (re.code&&re.code != 200) {
        throw "call error, code:" + re.code + ", message:" + re.result;
      }
    }catch(ex){
      // console.log(ex);
    }
    return re.result;
  }

  /**
   * 此处wii跟java代码调用实例名一致
   */
  if (isAndroid() && versionCompare(getVersion(), "1") >= 0) {
    declaredPublicMethods.forEach(function(d) {
      T[d] = function() {
        // 取参数
        var params = S.call(arguments, 0)
        var p = []
        if (typeof(params[0]) == 'object'){
          Object.getOwnPropertyNames(params[0]).forEach(function(key){
              var _p = params[0][key]
              if(typeof(_p) == 'object'){
                p.push(JSON.stringify(_p))
              }else{
                p.push(_p)
              }
          })
        }else if (typeof(params[0]) !== 'undefined'){
          p.push(params[0])
        }

        // 执行函数
        jsApi[d](...p)
        //eval("jsApi."+callFunction)
      //  return promptCall.apply(T, [d].concat(S.call(arguments, 0)));
      }
    });
    if(typeof context[nameSpace] == 'undefined'){
      window[nameSpace] = {}
    }
    context[nameSpace] = deepAssign(context[nameSpace], T);
  }

  if (isIOS() && versionCompare(getVersion(), "1") >= 0) {
    if (typeof context[nameSpace] == 'undefined') {
      context[nameSpace] = {};
    }

    /** 关闭当前窗口
     *
     */
    context[nameSpace].closeWindow = function() {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("closeWindow", {});
      }
    }

    /** 打开新窗口
     *
     */
    context[nameSpace].openWindow = function(windowInfo) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("openWindow", windowInfo);
      }
    }

    /** 设置title
     *
     */
    context[nameSpace].setTitle = function(title) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("setTitle", {
          'title': title
        });
      }
    }

    /** 地址信息
     *
     */
    context[nameSpace].locationInfo = function(locationInfo) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("locationInfo", locationInfo);
      }
    }

    /** 设置分享
     *
     */
    context[nameSpace].setActionBar = function(info) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("setActionBar", info);
      }
    }

    /** 登录
     *
     */
    context[nameSpace].login = function(loginInfo) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("login", loginInfo);
      }
    }

    /** 从 App 中获取数据
     *
     */
    context[nameSpace].getAppData = function(appInfo) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("getAppData", appInfo);
      }
    }

    /** 清除缓存
     *
     */
    context[nameSpace].clearCache = function(flag) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("clearCache", flag);
      }
    }

    context[nameSpace].alysis = function(data) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("alysis", data);
      }
    }

    /** 显示/隐藏加载动画
     *
     */
    context[nameSpace].showLoading = function(info) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("showLoading", info);
      }
    }

    context[nameSpace].sendAppData = function(data) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("sendAppData", data);
      }      
    }

    context[nameSpace].message = function(data) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("message", data);
      }
    }

    context[nameSpace].getGallery = function(data) {
      if (typeof jsBridge != 'undefined') {
        jsBridge.postNotification("getGallery", data);
      }
    }

  }

  context[nameSpace] = context[nameSpace]||{};

  context[nameSpace].isNotApp = function() {
//    return !navigator.userAgent.match(/yyn\/([\.\d]+)/i);
    return false
  }
  context[nameSpace].app = {};

  context[nameSpace].app.invoke = function(functionName, params, errorCallback) {
    // console.warn('---华丽的分割线---');
    if (!context[nameSpace].isNotApp()&&typeof wii[functionName] !== 'undefined') {
      // console.warn('name:' + functionName);
      // console.warn('params看下面');
      // console.warn(params);
      if (typeof params!=='undefined') {
        wii[functionName].call(this, params);
      } else {
        wii[functionName].call(this);
      }
    } else {
      console.error('没有找到方法：' + functionName);
      errorCallback ? errorCallback.apply(this, [functionName, params]) : '';
    }
  }

})(this, "wii"); /*  |xGv00|ce6cf96580b5124719bfd36450bef56f */